      program coating
      implicit none

! calculatines the volume of coating around an ellipsoid,
! where the coating is applied normal to the surface.


! standard form of ellipsoid: (x/a)^2 + (y/b)^2 + (z/c)^2 = 1
! volume of ellipoid = 4/3 * pi * a * b * c
! surface area of ellipsoid = 2*pi*c^2 + 2*pi*a*b/sin(phi) * [E(phi,k)*sin^2(phi) + F(phi,k)*cos^2(phi)]
! where cos(phi) = c/a 
! k^2 = a^2/b^2 * (b^2 - c^2)/(a^2 - c^2) 
! F(phi,k) is the legendre form ! of the first incomplete elliptic integral
! E(phi,k) is the legendre form of the second incomplete elliptic integral
! see https://en.wikipedia.org/wiki/Ellipsoid 


! apply steiner's formula for a growing shape in 3d:
! volume_new = volume_old + area * d + pi*mean_length * d^2 + 4/3*pi * d^3
! volume coating = volume_new - volume_old = area * d + pi * mean_length * d^2 + 4/3*pi * d^3
! the tricky part is the "mean length" or volume to surface area ratio, see below

! declare
      character*80      :: string
      real*8            :: a, b, c, d, ap, bp, cp, ain, bin, cin
      real*8            :: volume_old, area, area_term, mean_length, width_term, & 
                           volume_displace, volume_new, volume_coating, answer
      real*8, parameter :: pi = 3.14159265358979323846264338327950d0, &
                           tiny = 1.0d-20


! popular formats
01    format(1x,a23,1f14.8)


! get semi-major axes and coating thickness
10    string = ' ' 
      write(6,*) ' give semimajor axes a, b, c, and coating thickness d (hit return to exit) =>'
      read(5,'(a)') string
      if (string(1:4) .eq. '    ') stop
      read(string,*,err=10) ain, bin, cin, d
      a = ain ; b = bin ; c = cin
      if (a <= 0) a = tiny ; if (b <= 0) b = tiny  ; if (c <= 0) c = tiny
!     write(6,*) a,b,c
      write(6,*)


! volume and surface area of ellipsoid are independent of cyclic axes label permutations.
! this is equivalent to rotations that bring the ellipsoid to a preferred 
! set of axes, but rotations do not change volumes or surface areas
! ensure a >= b >= c by sorting

      call sort3(c,b,a)
!     write(6,*) a,b,c


! volume of original ellipsoid
      volume_old = 4.0d0/3.d0 * pi * a * b * c
      write(6,01) 'volume_old =           ',volume_old


! surface area term of steiner's formula 
      call surface_area_ellipsoid(a,b,c,area)
      area_term = area * d
      write(6,01) 'surface_area term =    ',area_term



! cauchy's surface area to volume formula for an ellipse
! S/V = 3 * integral (ax)^2 + (by)^2 + (cz)^2 dA  or
! mean_length = 3/(4*pi^2) * V * A(1/a,1/b,1/c) = a*b*c/pi * A(1/a,1/b,1/c)
! note the inverse semimajor aces are the radii of curvatures
! note for a sphere the mean length is 4*r

! cyclic permutation of axes labels to ensure inverses maintain a >= b >= c
      ap = 1.0d0/c   ; bp = 1.0d0/b   ;  cp = 1.0d0/a

      call surface_area_ellipsoid(ap,bp,cp,area)
      mean_length = a * b * c * area/pi 
      width_term = pi * mean_length * d * d
      write(6,01) 'width_term term =      ',width_term


! volume of displacement
      volume_displace = 4.0d0/3.d0 * pi * d * d * d
      write(6,01) 'vol_displacement term =',volume_displace


! new volume and coating volume
      volume_new = volume_old + area_term + width_term + volume_displace
      write(6,01) 'volume_new =           ',volume_new

      volume_coating = area_term + width_term + volume_displace
      write(6,01) 'volume_coating =       ',volume_coating


! some known answers
! https://www.johannesbader.ch/2013/12/project-euler-problem-449-chocolate-covered-candy/

      if ( (ain == 1.0) .and. (bin == 1.0) .and. (cin == 1.0) .and. (d == 1.0) ) then
       answer = 28.0d0/3.0d0 * pi
       write(6,01) 'known answer =         ',answer

      else if ( (ain == 1.0) .and. (bin == 1.0) .and. (cin == 1.0) .and. (d == 2.0) ) then
       answer = 104.0d0/3.0d0 * pi
       write(6,01) 'known answer =         ',answer

      else if ( (ain == 2.0) .and. (bin == 2.0) .and. (cin == 2.0) .and. (d == 1.0) ) then
       answer = 76.0d0/3.0d0 * pi
       write(6,01) 'known answer =         ',answer

      else if ( (ain == 2.0) .and. (bin == 2.0) .and. (cin == 1.0) .and. (d == 1.0) ) then
       answer = 60.35475635d0
       write(6,01) 'known answer =         ',answer

      else if ( (ain == 4.0) .and. (bin == 4.0) .and. (cin == 1.0) .and. (d == 1.0) ) then
       answer = 158.607408085576d0
       write(6,01) 'known answer =         ',answer

      else if ( (ain == 1.0) .and. (bin == 1.0) .and. (cin == 100.0) .and. (d == 3.0) ) then
       answer = 8731.988494314264d0
       write(6,01) 'known answer =         ',answer

      else if ( (ain == 7.0) .and. (bin == 7.0) .and. (cin == 9.0) .and. (d == 2.0) ) then
       answer = 1892.426068286629d0
       write(6,01) 'known answer =         ',answer

      else if ( (ain == 0.0) .and. (bin == 0.0) .and. (cin == 4.0) .and. (d == 1.0) ) then
       answer = 8.0d0 * pi
      end if
      write(6,*)

! go back for another
      goto 10

      end program coating




      subroutine surface_area_ellipsoid(a,b,c,area)
      implicit none

! calculates the surface area of a general triaxial ellipsoid
! must have a >= b >= c
! uses the legendre form of the elliptic intgerals
! see https://en.wikipedia.org/wiki/Ellipsoid

! declare the pass
      real*8            :: a,b,c,area

! local variables
      real*8            :: phi,denom,numer,ak,fac1,fac2,fac3,ellf,elle
      real*8, parameter :: pi = 3.14159265358979323846264338327950d0, & 
                           tiny = 1.0d-30

! angle phi in radians
      phi  = acos(c/a)
!      write(6,01) 'phi =',phi

! angle ak in radians - avoid division by zero to handle the case of a sphere
      denom = b*b * (a*a - c*c)
      denom = denom + tiny
      numer = a*a * (b*b - c*c)
      ak = sqrt(numer/denom)
!      write(6,01) 'ak =',ak

! surface area formula - avoid division by zero to handle the case of a sphere
      fac1 = 2.0d0 * pi * c*c
      fac2 = 2.0d0 * pi * a*b/(sin(phi) + tiny) 
      fac3 = ellf(phi,ak)*cos(phi)*cos(phi) + elle(phi,ak)*sin(phi)*sin(phi) + tiny
      area = fac1 + fac2 * fac3

      return
      end subroutine surface_area_ellipsoid



      subroutine sort3(x,y,z)  ! sort 3 numbers in ascending order
      implicit none
      real*8 :: x,y,z
      if (x > z) call swap(x,z)
      if (x > y) then
       call swap(x,y)
      else if (y > z) then
       call swap(y,z)
      end if
      end subroutine sort3

      subroutine swap(a,b)   ! exchange a and b
      implicit none
      real*8 :: a,b,x
      x = a ; a = b ; b = x
      return
      end subroutine swap






! elliptic functions:
! function rf is the first carlson elliptic integral
! function rd is the third degenerate carlson elliptic integral

! function ellf returns the legendgre elliptic integral of the first kind
! function elle returns the legendgre elliptic integral of the second kind




      real*8 function rf(x,y,z)
      implicit none

! computes carlson's elliptic integral of the first kind rf(x,y,z).
! x, y and z must be non-negative and at most one can be zero. the carlson
! function rf is very closely related to legendre's elliptic integral
! of the first, second and third kinds.
!
! the parameter tiny must be at most five times machine underflow and
! big must be at most one fifth the machine overflow. the limits here
! are conservative for double precision.
!
! declare the pass
      real*8            :: x,y,z

! declare locals
      real*8            :: alamb,ave,delx,dely,delz,e2,e3,sqrtx,sqrty,sqrtz,xt,yt,zt
      real*8, parameter :: errtol=0.0025d0, tiny=1.5d-38, big=3.0d37, &
                           third=1.0d0/3.0d0, c1=1.0d0/24.0d0, c2=0.1d0, &
                           c3 = 3.0d0/44.0d0, c4 = 1.0d0/14.0d0

! error check the input
      if (min(x,y,z) .lt. 0.0  .or.  min(x+y,x+z,y+z) .lt. tiny  .or. &
          max(x,y,z).gt.big) stop ' invalid arguments to function rf'

! go
      xt = x
      yt = y
      zt = z
01    continue
      sqrtx = sqrt(xt) 
      sqrty = sqrt(yt)
      sqrtz = sqrt(zt)
      alamb = sqrtx * (sqrty + sqrtz) + sqrty * sqrtz
      xt    = 0.25d0 * (xt + alamb)
      yt    = 0.25d0 * (yt + alamb)
      zt    = 0.25d0 * (zt + alamb)
      ave   = third * (xt + yt + zt)
      delx  = (ave-xt)/ave
      dely  = (ave-yt)/ave
      delz  = (ave-zt)/ave
      if (max(abs(delx),abs(dely),abs(delz)).gt.errtol) goto 01
      e2    = delx*dely - delz*delz
      e3    = delx*dely*delz
      rf    = (1.0d0 + (c1*e2 - c2 -c3*e3)*e2 + c4*e3)/sqrt(ave)
      return
      end function rf



      real*8 function rd(x,y,z)
      implicit none

! this routine computes carlson's degenerate elliptic integral of the third
! kind rd(x,y,z,p) = rj(x,y,z,z). x and y must be non-negative and at most
! one can be zero. the  carlson function rd is very closely related to
! legendre's elliptic integral of the first, second and third kinds.
!
! tiny must be at least twice the negative 2/3 power of machine overflow.
! big must be at most 0.1 times the negative 2/3 power of  machine underflow. 
! the limits here are conservative for double precision.

! declare the pass
      real*8            :: x,y,z

! declare local variables
      real*8            :: alamb,ave,delx,dely,delz,ea,eb,ec,ed,ee,fac, & 
                           sqrtx,sqrty,sqrtz,sum,xt,yt,zt
      real*8, parameter :: errtol=0.0015d0, tiny=1.0d-40, big=1.0d40, &
                           c1=3.0d0/14.0d0, c2=1.0d0/6.0d0, &
                           c3=9.0d0/22.0d0, c4=3.0d0/26.0d0, &
                           c5=0.25d0*c3,    c6=1.5d0*c4

! error check the input
      if (min(x,y).lt.0.0 .or. min(x+y,z).lt.tiny .or. &
          max(x,y,z).gt.big) stop 'invalid pass in routine rd'

! go
      xt  = x
      yt  = y
      zt  = z
      sum = 0.0d0
      fac = 1.0d0
01    continue
      sqrtx = sqrt(xt)
      sqrty = sqrt(yt)
      sqrtz = sqrt(zt)
      alamb = sqrtx * (sqrty + sqrtz) + sqrty * sqrtz
      sum   = sum + fac/(sqrtz * (zt + alamb))
      fac   = 0.25d0 * fac
      xt    = 0.25d0 * (xt + alamb)
      yt    = 0.25d0 * (yt + alamb)
      zt    = 0.25d0 * (zt + alamb)
      ave   = 0.2d0 * (xt + yt + 3.0d0*zt)
      delx  = (ave-xt)/ave
      dely  = (ave-yt)/ave
      delz  = (ave-zt)/ave
      if (max(abs(delx),abs(dely),abs(delz)).gt.errtol) goto 01
      ea = delx * dely
      eb = delz * delz
      ec = ea - eb
      ed = ea - 6.0d0 * eb
      ee = ed + ec + ec
      rd = 3.0d0*sum + fac*(1.0d0 + ed*(-c1 + c5*ed - c6*delz*ee) &
           + delz*(c2*ee + delz*(-c3*ec + delz*c4*ea)))/(ave*sqrt(ave))
      return
      end function rd





      real*8 function ellf(phi,ak)
      implicit none

! returns the legendre elliptic integral of the first kind.
! input is phi (in radians),
!          ak  (in radians, = sin(alpha) in abromowitz and stegun notation).

! declare the pass
      real*8            :: phi,ak

! declare local variables
      real*8            :: rf,x,y,c1,s1
      real*8, parameter :: z=1.0d0

      c1   = cos(phi)
      s1   = sin(phi)
      x    = c1 * c1
      y    = (1.0d0 - ak*s1) * (1.0d0 + ak*s1)
      ellf = s1 * rf(x,y,z)
      return
      end function ellf





      real*8 function elle(phi,ak)
      implicit none

! returns the legendre elliptic integrals of the second kind.
! input is phi (in radians),
!          ak  (in radians, = sin(alpha) in abromowitz and stegun notation).

! declare the pass
      real*8           :: phi,ak


! declare local varibales
      real*8           :: rf,rd,x,y,c1,s1,s2
      real*8,parameter :: z=1.0d0, third = 1.0d0/3.0d0

      c1   = cos(phi)
      s1   = sin(phi)
      s2   = s1 * s1
      x    = c1 * c1
      y    = (1.0d0 - ak*s1) * (1.0d0 + ak*s1)
      elle = s1 * ( rf(x,y,z) - third * ak*ak * s2 * rd(x,y,z) )
      return
      end function elle
